/*****************************************************************************
*   Gnome Wave Cleaner Version 0.01
*   Copyright (C) 2001 Jeffrey J. Welty
*   
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*******************************************************************************/

/* audio_util.c */

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/soundcard.h>
/*  #include <gtk/gtk.h>  */
/*  #include <rfftw.h>  */
/*  #include "gwc.h"  */
#include "fmtheaders.h"

#define NOTHING 0x00
#define MONITOR 0x01
#define RECORDING  0x02
#define PLAYBACK  0x04

int audio_fd = -1 ;
int audio_state = 0 ;
int wavefile_fd = -1 ;
long audio_bytes_written ;
int rate = 44100 ;
int stereo = 1 ;
int audio_bits = 16 ;
int bytes_per_sample = 2 ;
/*  int dump_sample = 0 ;  */
long wavefile_data_start ;

extern struct view audio_view ;

int current_sample ;

long playback_total_bytes = 0 ;
int playback_bytes ;

int verbose = 0 ;
int repair = 0 ;

struct sound_prefs {
    int bits ;
    int rate ;
    int stereo ;
    long n_samples ;
    int n_channels ;
    int wavefile_fd ;
    long max_value ;
    long max_allowed ;
    int sample_buffer_exists ;
    long data_length ;

    size_t mmap_size ;
    void *mmap_file ;
    size_t data_offset ;
    ulong data_size ;
    void *data ;
} ;

char *sample_to_time_text(long i, int rate, char *prefix, char *buf)
{
    int m, s, ms ;
    char sec_buf[10] ;

    m = i/(rate*60) ;
    i -= m*rate*60 ;

    s = i/rate ;
    i -= s*rate ;
    ms = 1000*i/rate ;

    if(ms > 500) s++ ;

    sprintf(sec_buf, "%2d", s) ;

    if(sec_buf[0] == ' ') sec_buf[0] = '0' ;

    sprintf(buf, "%s%d:%s", prefix, m,sec_buf) ;
    return buf ;
}

void print_wavefile(char *filename)
{
    int wavfile_fd ;
    char buf[200] ;
    struct sound_prefs wfh ;
    wavhead header;

    wavefile_fd = open(filename, O_RDWR) ;

    if( wavefile_fd == -1) {
	fprintf(stderr, "Failed to open  %s", filename) ;
	return ;
    }


    read(wavefile_fd, &header, sizeof(header));

    audio_bits = wfh.bits = header.bit_p_spl ;
    bytes_per_sample = wfh.bits / 8 ;
    wfh.rate = header.sample_fq ;
    wfh.n_channels = header.modus ;
    wfh.stereo = stereo = header.modus - 1 ;
    wfh.n_samples = header.data_length / bytes_per_sample / wfh.n_channels ;
    wfh.data_length = header.data_length ;

    wfh.data_offset = wavefile_data_start = sizeof(header) ;
    printf("%s\t%s\n", filename, 
	sample_to_time_text(wfh.n_samples, wfh.rate, "", buf)) ;

    if(repair) {
	printf("pre-repair sample_fq:%uld\n", header.sample_fq) ;
	header.sample_fq >>= 1 ;
	header.sample_fq <<= 1 ;
	header.byte_p_sec = header.sample_fq * header.byte_p_spl ;

	lseek(wavefile_fd, 0, SEEK_SET) ;
	write(wavefile_fd, &header, sizeof(header));
    }

    close(wavefile_fd) ;

    if(verbose) {
	printf("main_chunk:%uld\n", header.main_chunk) ;
	printf("length:%uld\n", header.length) ;
	printf("chunk_type:%uld\n", header.chunk_type) ;
	printf("sub_chunk:%uld\n", header.sub_chunk) ;
	printf("sc_length:%uld\n", header.sc_len) ;

	printf("format:%ud\n", header.format) ;
	printf("modus:%ud\n", header.modus) ;

	printf("sample_fq:%uld\n", header.sample_fq) ;
	printf("byte_p_sec:%uld\n", header.byte_p_sec) ;

	printf("bytes_per_sample:%ud\n", header.byte_p_spl) ;
	printf("bits_per_sample:%ud\n", header.bit_p_spl) ;

	printf("data_chunk:%uld\n", header.data_chunk) ;
	printf("data_length:%uld\n", header.data_length) ;
    }

#ifdef THIS_IS_THE_HEADER_FORMAT_SAMPLE
typedef struct wavhead {
  u_long	main_chunk;	/* 'RIFF' */
  u_long	length;		/* Length of rest of file */
  u_long	chunk_type;	/* 'WAVE' */

  u_long	sub_chunk;	/* 'fmt ' */
  u_long	sc_len;		/* length of sub_chunk, =16 (rest of chunk) */
  u_short	format;		/* should be 1 for PCM-code */
  u_short	modus;		/* 1 Mono, 2 Stereo */
  u_long	sample_fq;	/* frequence of sample */
  u_long	byte_p_sec;
  u_short	byte_p_spl;	/* samplesize; 1 or 2 bytes */
  u_short	bit_p_spl;	/* 8, 12 or 16 bit */ 

  u_long	data_chunk;	/* 'data' */
  u_long	data_length;	/* samplecount (lenth of rest of block?)*/
} wavhead;

#endif
}

int main(int argc, char **argv)
{
    int i = 1 ;
    verbose = 0 ;
    repair = 0 ;


    while(i < argc) {
	if(argv[i][0] == '-') {
	    if(!strcmp("-a", argv[i])) verbose = 1 ;
	    if(!strcmp("-r", argv[i])) repair = 1 ;
	}
	i++ ;
    }


    for(; i < argc ; i++)
	print_wavefile(argv[i]) ;
}
